@@ -26,7 +26,6 @@ import java.util.List;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 public class BuildExperimentRunner {
-    private final GCLoggingCollector gcCollector = new GCLoggingCollector();
     private final DataCollector dataCollector;
     private final GradleSessionProvider executerProvider;
     private final OperationTimer timer = new OperationTimer();
@@ -41,6 +40,7 @@ public class BuildExperimentRunner {
         MemoryInfoCollector memoryInfoCollector = new MemoryInfoCollector();
         memoryInfoCollector.setOutputFileName("build/totalMemoryUsed.txt");
         BuildEventTimestampCollector buildEventTimestampCollector = new BuildEventTimestampCollector("build/buildEventTimestamps.txt");
+        GCLoggingCollector gcCollector = new GCLoggingCollector();
         dataCollector = new CompositeDataCollector(memoryInfoCollector, gcCollector, buildEventTimestampCollector, new CompilationLoggingCollector());
     }
 
@@ -63,28 +63,52 @@ public class BuildExperimentRunner {
 
         session.prepare();
         try {
-            for (int i = 0; i < experiment.getWarmUpCount(); i++) {
+            int warmUpCount = warmupsForExperiment(experiment);
+            for (int i = 0; i < warmUpCount; i++) {
                 System.out.println();
                 System.out.println(String.format("Warm-up #%s", i + 1));
-                runOnce(session, experiment, new MeasuredOperationList(), projectDir, Phase.WARMUP, i + 1, experiment.getWarmUpCount());
+                runOnce(session, experiment, new MeasuredOperationList(), projectDir, Phase.WARMUP, i + 1, warmUpCount);
             }
-            waitForMillis(experiment.getSleepAfterWarmUpMillis());
-            for (int i = 0; i < experiment.getInvocationCount(); i++) {
+            waitForMillis(experiment, experiment.getSleepAfterWarmUpMillis());
+            int invocationCount = invocationsForExperiment(experiment);
+            for (int i = 0; i < invocationCount; i++) {
                 if (i > 0) {
-                    waitForMillis(experiment.getSleepAfterTestRoundMillis());
+                    waitForMillis(experiment, experiment.getSleepAfterTestRoundMillis());
                 }
                 System.out.println();
                 System.out.println(String.format("Test run #%s", i + 1));
-                runOnce(session, experiment, results, projectDir, Phase.MEASUREMENT, i + 1, experiment.getInvocationCount());
+                runOnce(session, experiment, results, projectDir, Phase.MEASUREMENT, i + 1, invocationCount);
             }
         } finally {
             session.cleanup();
         }
     }
 
+    private Integer invocationsForExperiment(BuildExperimentSpec experiment) {
+        if (experiment.getInvocationCount() != null) {
+            return experiment.getInvocationCount();
+        }
+        // Take more samples when using the daemon, as execution time tends to be spiky
+        if (experiment.getInvocation().getBuildWillRunInDaemon()) {
+            return 8;
+        }
+        return 5;
+    }
+
+    private int warmupsForExperiment(BuildExperimentSpec experiment) {
+        if (experiment.getWarmUpCount() != null) {
+            return experiment.getWarmUpCount();
+        }
+        // Use more invocations to warmup when using the daemon, to allow the JVM to warm things up
+        if (experiment.getInvocation().getBuildWillRunInDaemon()) {
+            return 3;
+        }
+        return 1;
+    }
+
     // the JIT compiler seems to wait for idle period before compiling
-    private void waitForMillis(Long sleepTimeMillis) {
-        if (sleepTimeMillis > 0L) {
+    private void waitForMillis(BuildExperimentSpec experiment, long sleepTimeMillis) {
+        if (experiment.getInvocation().getBuildWillRunInDaemon() && sleepTimeMillis > 0L) {
             System.out.println();
             System.out.println(String.format("Waiting %d ms", sleepTimeMillis));
             try {
@@ -126,14 +150,9 @@ public class BuildExperimentRunner {
         final Runnable runner = session.runner(new GradleInvocationCustomizer() {
             @Override
             public GradleInvocationSpec customize(GradleInvocationSpec invocationSpec) {
-                GradleInvocationSpec customizedInvocation = invocationSpec.withAdditionalArgs(createIterationInfoArguments(phase, iterationNumber, iterationMax));
-                if (experiment.getListener() != null) {
-                    GradleInvocationCustomizer customizer = experiment.getListener().createInvocationCustomizer(invocationInfo);
-                    if (customizer != null) {
-                        customizedInvocation = customizer.customize(customizedInvocation);
-                    }
-                }
-                return customizedInvocation;
+                GradleInvocationSpec gradleInvocationSpec = invocationSpec.withAdditionalArgs(createIterationInfoArguments(phase, iterationNumber, iterationMax));
+                System.out.println("Run Gradle using JVM opts: " + gradleInvocationSpec.getJvmOpts());
+                return gradleInvocationSpec;
             }
         });
 
