@@ -17,6 +17,10 @@
 package org.gradle.play.internal;
 
 import org.gradle.api.file.FileCollection;
+import org.gradle.api.internal.file.FileResolver;
+import org.gradle.jvm.internal.WithJvmAssembly;
+import org.gradle.language.base.LanguageSourceSet;
+import org.gradle.language.scala.internal.ScalaJvmAssembly;
 import org.gradle.platform.base.internal.BinarySpecInternal;
 import org.gradle.play.PlayApplicationBinarySpec;
 import org.gradle.play.internal.toolchain.PlayToolChainInternal;
@@ -24,7 +28,7 @@ import org.gradle.play.platform.PlayPlatform;
 
 import java.io.File;
 
-public interface PlayApplicationBinarySpecInternal extends PlayApplicationBinarySpec, BinarySpecInternal {
+public interface PlayApplicationBinarySpecInternal extends PlayApplicationBinarySpec, BinarySpecInternal, WithJvmAssembly {
     void setTargetPlatform(PlayPlatform platform);
 
     void setToolChain(PlayToolChainInternal toolChain);
@@ -38,4 +42,9 @@ public interface PlayApplicationBinarySpecInternal extends PlayApplicationBinary
     FileCollection getClasspath();
 
     void setClasspath(FileCollection applicationClasspath);
+
+    ScalaJvmAssembly getAssembly();
+
+    void addGeneratedScala(LanguageSourceSet input, FileResolver fileResolver);
+    void addGeneratedJavaScript(LanguageSourceSet input, FileResolver fileResolver);
 }
